-- load libraries
local YakHelpers = AceLibrary("YakHelpers-2.0")
local YakScheduler = AceLibrary("YakScheduler-2.0")
local Waterfall = AceLibrary("Waterfall-1.0")
local Tablet = AceLibrary("Tablet-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Casualty")
local Roster = AceLibrary("Roster-2.1")

local options = {
   type = 'group',
   args = {
      Checks = {
	 type = 'group',
	 desc = 'Checks',
	 name = 'Checks',
	 args = {
	    CheckPartyRaidonly = {
	       type = 'toggle',
	       name = 'Party and Raid Only',
	       desc = 'Check only for casualties in current party/raid',
	       get = function()
			return CASUALTY.db.profile.CheckPartyRaidOnly
		     end,
	       set = function (b)
			CASUALTY.db.profile.CheckPartyRaidOnly = b
		     end
	    },
	    CheckPets = {
	       type = 'toggle',
	       name = 'Pets',
	       desc = 'Check for pets as well as players',
	       get = function()
			return CASUALTY.db.profile.CheckPets
		     end,
	       set = function(b)
			CASUALTY.db.profile.CheckPets = b
		     end
	    },
	    CheckDestroyedObjects = {
	       type = 'toggle',
	       name = 'Destroyed Objects',
	       desc = 'Check for destroyed objects (shaman totems, etc)',
	       get = function()
			return CASUALTY.db.profile.CheckDestroyedObjects
		     end,
	       set = function (b)
			CASUALTY.db.profile.CheckDestroyedObjects = b
		     end
	    },
	 },
      },
      Announcements = {
	 type = 'group',
	 name = 'Announcements',
	 desc = 'Types of announcements',
	 args = { 
	    Enabled = {
	       type = 'toggle',
	       name = 'Enable Announcements',
	       desc = 'Toggle for announcements on/off',
	       get = function()
			return CASUALTY.db.profile.Enabled
		     end,
	       set = function( b ) 
			CASUALTY.db.profile.Enabled = b
		     end
	    },
	    VoiceEnabled = {
	       type = 'toggle',
	       name = 'Voice Announcements',
	       desc = 'Enable voice announcements',
	       get = function()
			return CASUALTY.db.profile.VoiceEnabled
		     end,
	       set = function (b)
			CASUALTY.db.profile.VoiceEnabled = b
		     end
	    },
	    EnableMassCasualties = {
	       type = 'toggle',
	       name = 'Mass Casualties',
	       desc = 'Enable Mass Casualties announcement',
	       get = function()
			return CASUALTY.db.profile.EnableMassCasualties
		     end,
	       set = function(b)
			CASUALTY.db.profile.EnableMassCasualties = b
		     end
	    },
	    EnableNoSurvivors = {
	       type = 'toggle',
	       name = 'No Survivors',
	       desc = 'Enable No Survivors announcement',
	       get = function()
			return CASUALTY.db.profile.EnableNoSurvivors
		     end,
	       set = function(b)
			CASUALTY.db.profile.EnableNoSurvivors = b
		     end
	    },
	    UseSCT = {
	       type = 'toggle',
	       name = 'SCT Announcements',
	       desc = 'Use Scrolling Combat Text for announcements',
	       get = function()
			return CASUALTY.db.profile.UseSCT
		     end,
	       set = function(b)
			CASUALTY.db.profile.UseSCT = b
		     end
	    },
	 },
      },
      Delay = {
	 type = 'range',
	 name = 'Delay',
	 desc = 'Minimum delay between announcements',
	 min = 0,
	 max = 5,
	 step = 1,
	 get = function()
		  return CASUALTY.db.profile.Delay
	       end,
	 set = function ( i )
		  CASUALTY.db.profile.Enabled = i
	       end
      },
      SpeechDelay = {
	 type = 'range',
	 name = 'SpeechDelay',
	 desc = 'Min delay between speech announcements',
	 min = 0,
	 max = 10,
	 step = 1,
	 get = function()
		  return CASUALTY.db.profile.SpeechDelay
	       end,
	 set = function(i)
		  CASUALTY.config.SpeechDelay = i
	       end
      },
      MassCasualtiesLimit = {
	 type = 'range',
	 name = 'MassCasualtiesLimit',
	 desc = 'Number of simultaneous deaths to trigger Mass Casualties',
	 min = 1,
	 max = 40,
	 step = 1,
	 get = function()
		  return CASUALTY.db.profile.MassCasualtiesLimit
	       end,
	 set = function(i)
		  CASUALTY.db.profile.MassCasualtiesLimit = i
	       end
      },
      MessageColor = {
	 type = 'color',
	 name = 'MessageColor',
	 desc = 'Color to use for announcements in chat window',
	 hasAlpha = false,
	 get = function()
		  return  unpack(CASUALTY.db.profile.MessageColor)
	       end,
	 set = function(r,g,b)
		  CASUALTY.db.profile.MessageColor = {r,g,b}
	       end
      },
      SCTColor = {
	 type = 'color',
	 name = 'SCTColor',
	 desc = 'Color to use for SCT announcements',
	 hasAlpha = false,
	 get = function()
		  return unpack(CASUALTY.db.profile.SCTColor)
	       end,
	 set = function(r,g,b)
		  CASUALTY.db.profile.SCTColor = {r,g,b}
	       end
      },
      test = {
	 type = 'execute',
	 name = 'Test',
	 desc = 'Test an announcement using current settings',
	 func = function()
		   CASUALTY:Test()
		end
      },
      config = {
	 type = 'execute',
	 name = 'Config',
	 desc = 'Open Waterfall configuration menu',
	 wfHidden = true,
	 func = function()
		   CASUALTY:Config()
		end
      }
   }
}

local CASUALTY_SOUND_PATHS = {
   ["Casualty"] = "Interface\\Addons\\Casualty\\Sounds\\casualty.wav",
   ["Casualties" ] = "Interface\\Addons\\Casualty\\Sounds\\casualties.wav",
   ["MassCasualties"] = "Interface\\Addons\\Casualty\\Sounds\\mass_casualties.wav",
   ["NoSurvivors"] = "Interface\\Addons\\Casualty\\Sounds\\no_survivors.wav"
}

CASUALTY = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "Sink-1.0")
CASUALTY.hasNoColor = true
CASUALTY.clickableTooltip = true
CASUALTY.OnMenuRequest = options
CASUALTY.hideWithoutStandby = true
CASUALTY:RegisterChatCommand(L["SlashCommands"], options)

CASUALTY:RegisterDB("CasualtyDB", "CasualtyDBProf")
CASUALTY:RegisterDefaults("profile", {
			     ["Delay"] = 1,
			     ["SpeechDelay"] = 2,
			     ["Enabled"] = true,
			     ["VoiceEnabled"] = true,
			     ["CheckPartyRaidOnly"] = true,
			     ["CheckDestroyedObjects"] = true,
			     ["CheckPets"] = true,
			     ["MassCasualtiesLimit"] = 8,
			     ["EnableMassCasualties"] = true,
			     ["EnableNoSurvivors"] = true,
			     ["UseSCT"] = true,
			     ["MessageColor"] = {r=1,g=1,b=1},
			     ["SCTColor"] = {["r"]=1, ["g"]=1, ["b"]=1}
			  })
Waterfall:Register("Casualty", "aceOptions", options, "colorR", 0.6, "colorG", 0.6, "colorB", 0.6)

function CASUALTY:OnLoad()
  -- Called when addon is loaded
  CASUALTY.casualties = {}
  CASUALTY.NoSurvivorsReported = false
end

function CASUALTY:OnClick()
   Waterfall:Open("Casualty")
end

function CASUALTY:OnTooltipUpdate()
   Tablet:SetHint(L["Right-click for dewdrop configuration; left-click for waterfall configuration"])
end

function CASUALTY:OnEnable()
   self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function CASUALTY:OnDisable()
  -- called when addon is disabled
end

function CASUALTY:ToggleEnabled()
   self.db.profile.Enabled = self.db.profile.Enabled == false
   if self.db.profile.Enabled then
      self:Print("Reporting enabled")
   else
      self:Print("Reporting disabled")
      self.casualties = {}
   end
end

function CASUALTY:Config()
   CasualtyConfig:Show()
end


function CASUALTY:matches_message( arg, template )
   -- returns whether or not the string matches the template
   -- and as a second return, the name of the object matched
   local msg = string.gsub(template, "%%s", "(.-)")
   -- following gyrations are to match the empty message, particularly with
   -- PARTYKILLOTHER
   local emptymsg = string.gsub(template, "%%s", "",1)
   emptymsg = string.gsub(emptymsg, "%%s", "(.-)")
   local beginpos, endpos = string.find(arg,msg)
   if beginpos then
      local nbegin, nend = string.find(arg, emptymsg)
--      YakHelpers:print("arg: "..arg.."; beginpos: "..nbegin.."; name: "..string.sub(arg,1,nbegin-1))
--      YakHelpers:print("true:"..string.sub(arg,1,nbegin-1))
      return true, string.sub(arg, 1, nbegin-1)
   else
--      YakHelpers:print("false")
      return false
   end
end

function CASUALTY:log_death( name )
   if self.casualties == nil then
      self.casualties = {}
   end
   if table.getn(self.casualties) == 0 then
      YakScheduler.Scheduler:In(self.db.profile.Delay, "CASUALTY.Report", self.Report)
   end
   table.insert( self.casualties, name )
end

function CASUALTY:log_death_if_matched(arg, template)
   local matched, name = self:matches_message(arg, template)
   --   if matched then YakHelpers:print("matched "..name) else YakHelpers:print("not matched") end
--   YakHelpers:print(tostring(matched) .. ":" .. tostring(name))
--   YakHelpers:print(tostring(CASUALTY:IsReportableCasualty(name)))
   if matched and self:IsReportableCasualty(name) then
--      YakHelpers:print("adding casualty: ".. name)
--      if self.Timeout then
      self:log_death(name)
   end
end

function CASUALTY:NamedUnitIsInPartyOrRaid( name, checkpets )
   local result = false
--   self:Print("Checking if "..name.." is in party/raid")
   if name == nil then return false end
   local ob = Roster:GetUnitObjectFromName( name )
--   self:Print("Got roster object for "..name)
   if ob == nil then return false end
   if (checkpets == false) and (ob.class == 'PET') then return false end
--   self:Print(name .. " is in party/raid")
   return true
   -- got a strange usage question where this was called with some weird name
   -- YakHelpers:print("Checking unit named "..name)
--   if YakHelpers:PlayerIsInRaid() then
--      result = YakHelpers:NamedUnitIsInRaid(name, checkpets)
--   elseif YakHelpers:PlayerIsInGroup() then
--      result = YakHelpers:NamedUnitIsInParty(name, checkpets)
--   end
--   return result
end

function CASUALTY:IsReportableCasualty(name)
   -- note: objects (totems) are checked earlier
   local result = true
   if self.db.profile.CheckPartyRaidOnly then
--      self:Print("Checking party/raid")
      result = result and self:NamedUnitIsInPartyOrRaid(name, self.db.profile.CheckPets)
   end
   -- if it's a pet but not in our party, we won't display it
--   YakHelpers:print(name .. ":"..tostring(result))
   return result
end

function CASUALTY:COMBAT_LOG_EVENT_UNFILTERED(timestamp, event, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, ...)
   if (event ~= "UNIT_DIED") then return end
--   self:Print("Saw a death: "..destName)
   if self.db.profile.Enabled then
--      self:Print("enabled")
      -- look at the argument to see if it's something we want to match
      self:log_death_if_matched(destName, PARTYKILLOTHER)
      if self.db.profile.CheckDestroyedObjects then
--	 YakHelpers:print("Checking destroyed object")
	 self:log_death_if_matched(destName, UNITDESTROYEDOTHER)
      end
      self:log_death_if_matched(destName, UNITDIESOTHER)
      --   CASUALTY.log_death_if_matched(arg, UNITDIESSELF)
   end
end


function CASUALTY:AnySurvivors()
   local result = false
   if YakHelpers:PlayerIsSolo() then
      if not(UnitIsDead("player") or UnitIsGhost("player")) then
	 result = true
      end
   elseif YakHelpers:PlayerIsInRaid() then
      for i,id in YakHelpers:eachRaidID() do
	 if not (UnitIsDead(id) or UnitIsGhost(id)) then
	    result = true
	 end
      end
   elseif YakHelpers:PlayerIsInGroup() then
      for i, id in YakHelpers:eachPartyID() do
	 if not (UnitIsDead(id) or UnitIsGhost(id)) then
	    result = true
	 end
      end
   end
   return result
end

function CASUALTY:Report()
   -- note: fully qualifying CASUALTY instead of 'self' so that this can be
   -- called externally from YakScheduler
   -- Reset the no survivors message
   if CASUALTY:AnySurvivors() then
      CASUALTY.NoSurvivorsReported = false
   end
   if CASUALTY.db.profile.Enabled then
      -- from worst case to best
      if CASUALTY.db.profile.EnableNoSurvivors and (not CASUALTY:AnySurvivors()) and (not CASUALTY.NoSurvivorsReported) then
	 CASUALTY:ReportNoSurvivors()
      elseif CASUALTY.db.profile.EnableMassCasualties and (table.getn(CASUALTY.casualties) >= CASUALTY.db.profile.MassCasualtiesLimit) then
	 CASUALTY:ReportMassCasualties()
      elseif table.getn(CASUALTY.casualties) > 1 then
	 CASUALTY:ReportCasualties()
      elseif table.getn(CASUALTY.casualties) == 1 then
	 CASUALTY:ReportCasualty()
      end
   end
end
   

function CASUALTY:ShowCasualtiesMessage()
   local msg = L["Casualty"] .. ": "
   if table.getn(self.casualties) > 1 then
      msg = L["Casualties"] .. ": "
   end
   for k,v in ipairs(self.casualties) do
      msg = msg .. v..", "
   end
   msg = string.sub(msg, 1, -3)
   -- self:Pour("Lobter", 1,1,1, nil, nil, nil, false)
   DEFAULT_CHAT_FRAME:AddMessage(msg, self.db.profile.MessageColor[1], self.db.profile.MessageColor[2], self.db.profile.MessageColor[3] )
   if (self.db.profile.UseSCT == true) then
      local sct = getglobal("SCT")
      if sct ~= nil then
	 local sctcolor = { r = self.db.profile.SCTColor[1], g = self.db.profile.SCTColor[2], b = self.db.profile.SCTColor[3]}
	 sct:DisplayText(msg, sctcolor, nil, "event", 1)
      end
   end
end

function CASUALTY:PlayAudioReport(soundfile)
   if self.db.profile.VoiceEnabled then
      PlaySoundFile(soundfile)
   end
   self.casualties = {}
   YakScheduler.Scheduler:In(self.db.profile.SpeechDelay, "CASUALTY.Report", CASUALTY.Report)
end

function CASUALTY:ReportNoSurvivors()
   self:ShowCasualtiesMessage()
   DEFAULT_CHAT_FRAME:AddMessage(L["Casualties"] .. ": No Survivors")
   self:PlayAudioReport(CASUALTY_SOUND_PATHS.NoSurvivors)
   self.NoSurvivorsReported = true
end

function CASUALTY:ReportMassCasualties()
   self:ShowCasualtiesMessage()
   self:PlayAudioReport(CASUALTY_SOUND_PATHS.MassCasualties)
end

function CASUALTY:ReportCasualty()
   self:ShowCasualtiesMessage()
   self:PlayAudioReport(CASUALTY_SOUND_PATHS.Casualty)
end

function CASUALTY:ReportCasualties()
   self:ShowCasualtiesMessage()
   self:PlayAudioReport(CASUALTY_SOUND_PATHS.Casualties)
end

function CASUALTY:Config()
   Waterfall:Open("Casualty")
end
				
function CASUALTY:Test()
   self.casualties = {"Bob", "Tim"}
   self:Report()
end

